/*
 *    TODO.java
 *    Copyright (C) 2000 Stuart Yeates
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// the package we're in
package org.nzdl.gsdl.util;

// java standard library classes used
import java.io.IOException;
import java.io.Serializable;
import java.lang.Cloneable;
import java.util.Properties;
import java.util.Vector;
import java.util.Enumeration;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import org.omg.CORBA.Object;
import org.omg.CORBA.ORB;
import java.net.URL;

import org.nzdl.gsdl.service.*;

/**
 * Used to get host information including: name, IOR and URL.
 *
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @author  George Buchanan (g.buchanan@mdx.ac.uk)
 * @version $Revision: 2163 $
 * @see org.nzdl.gsdl.SimpleClient
 * @see org.nzdl.gsdl.SimpleServer
 *
 */

public class NzdlHosts implements Cloneable, Serializable {
  
  static String propertiesFileName = "NZDLServers.properties";

  static String defaultHostName = "nzdl2.cs.waikato.ac.nz";

  protected Properties table;

  private static NzdlHosts singleton = new NzdlHosts();
  
  private NzdlHosts()
  {
    table = new Properties();
    
    try {
      File file = new File(propertiesFileName);
      if (file.exists() && file.canRead())
	table.load(new FileInputStream(file));
      else {
	table.put("mdxlite", "ickenham.mdx.ac.uk");
	table.put("nzdl2", "nzdl2.cs.waikato.ac.nz");
	table.put("csdl1", "csdl1.mdx.ac.uk");
      }
    } catch (IOException exception) {
      System.err.println("NzdlHosts::NzdlHosts() error reading properties file: " + 
			 exception);
    }
  }

  public void finalize() {
    tidy();
  }

  static public void tidy() {
    if (singleton == null)
      singleton = new NzdlHosts();
    try {
      File file = new File(propertiesFileName);
      if (file.canWrite() || !file.exists())
	singleton.table.store(new FileOutputStream(file),"NZDL hosts we know about");
      else {
	System.err.println("NzdlHosts::tidy() unable ");
      }
    } catch (IOException exception) {
      System.err.println("NzdlHosts::NzdlHosts() error reading properties file: " + 
			 exception);
    }
  }
  

  static public void addHost(String code, String url)
  {
    if (singleton == null)
      singleton = new NzdlHosts();
    singleton.table.put(code, url);
  }


  static public String getFullName(String code)
  {
    if (singleton == null)
      singleton = new NzdlHosts();
    String reply;
    reply = (String) singleton.table.get(code);
    System.out.println("Read " + reply);
    if (reply == null)
      {       
	return defaultHostName;
      }
    return reply;
  }

  static public String getUrl(String code)
  {
    return "http://" + getHost(code) + "/cgi-bin/getior";
  }

  static public String getIOR(String code)
  {
    if (singleton == null)
      singleton = new NzdlHosts();

    String str = getUrl(code);
    String ior = null;

    try { 
      URL url = new URL(str);
      InputStream his = url.openStream();
      DataInputStream dhis = new DataInputStream(his) ;
      ior = dhis.readLine();
      return ior;
    } catch (Throwable throwable) {
      System.err.println("NzdlHosts::getIOR() unable to construct or read URL \"" +
			 str + "\", \"" + ior + "\": " + throwable);
      return "";
    }
  }

  static public NzdlService getHost(String code) {
    NzdlService nzdl = null;

    try {
      nzdl = new NzdlServiceClient(null,null,getIOR(code));
    } catch (Throwable throwable) {
      System.err.println("NzdlHosts::getHost() constructing NzdlServiceClient(" + 
			   code + ") " + throwable);
    }
    return nzdl;
  }

  static public Vector getAllHosts() {
    if (singleton == null)
      singleton = new NzdlHosts();

    Vector results = new Vector();

    Enumeration e = singleton.table.keys();
    while (e.hasMoreElements()) {
      String code = (String) e.nextElement();
      try {
	NzdlService nzdl = new NzdlServiceClient(null,null,getIOR(code));
	results.addElement(nzdl);
      } catch (Throwable throwable) {
	System.err.println("NzdlHosts::getHost() constructing NzdlServiceClient(" + 
			   code + "): " + throwable);
      }
    }
    return results;
  }

  static public NzdlService hostFromCommandlineArgs(String [] _args, Properties _props) {
    NzdlService nzdl = null;

    try {
      nzdl = new NzdlServiceClient( _args, _props, null);
    } catch (Throwable throwable) {
      System.err.println("NzdlHosts::hostFromCommandlineArgs() constructing " +
			 "NzdlServiceClient() " + throwable);
      
    }
    return nzdl;
  }
}



